@@ -3,26 +3,27 @@ from abc import ABC
 from botty_core.types import PTBHandler, Query
 from telegram import ext
 
+from botty.buttons import CallbackButton
 from botty.errors import CallbackDataError
-from botty.helpers import listify
 
 from .update import UpdateHandler
 
 
 class QueryHandler(UpdateHandler, ABC):
-    def __init__(self, button: str | list[str] | None = None) -> None:
-        self.on_button = button
+    def __init__(self, button: CallbackButton | None = None) -> None:
+        callback_data = None if button is None else button.callback_data
+        validate_callback_data(callback_data)
+        self.on_callback_data = callback_data
         super().__init__()
 
     def build(self) -> PTBHandler:
         return ext.CallbackQueryHandler(self.handle, self._filter)
 
     def _filter(self, callback_data: object) -> bool:
-        if self.on_button is None:
+        validate_callback_data(callback_data)
+        if self.on_callback_data is None:
             return True
-        if not isinstance(callback_data, str):
-            raise CallbackDataError(callback_data)
-        return callback_data in listify(self.on_button)
+        return callback_data == self.on_callback_data
 
     async def answer(self, text: str, *, show_alert: bool = False) -> bool:
         return await self.query.answer(text, show_alert=show_alert)
@@ -30,3 +31,8 @@ class QueryHandler(UpdateHandler, ABC):
     @property
     def query(self) -> Query:
         return self.update.query
+
+
+def validate_callback_data(callback_data: object) -> None:
+    if not isinstance(callback_data, str):
+        raise CallbackDataError(callback_data)
